@@ -40,6 +40,7 @@ import pycurl
 import exceptions
 import re
 import urllib
+import tempfile
 from bombardier_core.static_data import OK, FAIL
 from Exceptions import UnexpectedDataException, ServerException
 from Exceptions import MachineTraceback, ServerTracebackException
@@ -48,7 +49,30 @@ import libUi
 from SystemStateSingleton import SystemState
 system_state = SystemState()
 
-LOGIN_PATH = "/accounts/login/"
+NULL_POST = {"spam": "eggs"}
+LOGIN_PATH = "accounts/login/"
+SEARCH_PATH = "json/%s/search/"
+NAME_PATH = "json/%s/name/%s"
+MACHINE_PATH = "json/machine/name/%s"
+PACKAGE_SEARCH_PATH = "json/package/search/%s"
+PACKAGE_NAME_PATH = "json/package/name/%s"
+SUMMARY_NAME_PATH = "json/summary/name/%s"
+URL_LOOKUP = {'test': "json/machine/start_test/%s",
+              'dist': "json/machine/dist/%s",
+              'init': "json/machine/init/%s",
+              'reconcile': "json/machine/reconcile/%s",
+              'check-status': "json/machine/check_status/%s",
+              'fix': "json/machine/status/%s",
+              'purge': "json/machine/status/%s",
+              'enable': "json/machine/enable/%s",
+              'disable': "json/machine/disable/%s",
+              'setup': "json/machine/setup/%s",
+             }
+
+
+def curl_debug(debug_type, debug_msg):
+    "Interface for debugging curl commands"
+    print "debug(%d): %s" % (debug_type, debug_msg.strip())
 
 def make_query_string(args):
     """args -- a dictionary.
@@ -127,7 +151,12 @@ class Response:
                     msg = "can't convert to yaml (%s)" % self.output
                     raise UnexpectedDataException(msg)
             if type(output) != type([]) and type(output) != type({}):
-                raise UnexpectedDataException("Not a list or dictionary")
+                _fd, file_name = tempfile.mkstemp(suffix=".yml", text=True)
+                handle = open(file_name, 'w')
+                handle.write(str(self.output))
+                handle.close()
+                msg = "Not a list or dictionary (written to %s)" % file_name
+                raise UnexpectedDataException(msg)
             if output == None:
                 raise UnexpectedDataException("Null value received")
             if 'traceback' in output:
@@ -147,6 +176,9 @@ class CnmConnector:
         address -- URL of the Bombardier web server
         username -- username for connecting to the server'''
         self.break_handler = BreakHandler.BreakHandler()
+        if not address.endswith('/'):
+            address = "%s/" % address
+            #address = address[:-1]
         self.address = address
         self.username = username
         self.password = None
@@ -195,6 +227,8 @@ class CnmConnector:
         response = Response(header, output)
         if self.debug:
             print "PERFORMING:", curl_obj
+            curl_obj.setopt(pycurl.VERBOSE, 1)
+            curl_obj.setopt(pycurl.DEBUGFUNCTION, curl_debug)
         try:
             self.break_handler.enable()
             curl_obj.perform()
@@ -212,6 +246,7 @@ class CnmConnector:
         data -- dictionary of POST data
         '''
         full_path = urlparse.urljoin(self.address, path)
+        #full_path = '/'.join(self.address, path)
         curl_obj = self.prepare_curl_object(full_path)
         post_data = []
         for key in data:
@@ -222,16 +257,49 @@ class CnmConnector:
             print "POSTING:", encoded_post_data
         return self.perform_request(curl_obj, full_path)
 
+    def get_server_version(self):
+        url = "json/version"
+        output = self.service_yaml_request(url)
+        return output
+
+    def delete_user(self, user_name):
+        "Remove an administrative user from the system"
+        url = "json/user/name/%s" % user_name
+        output = self.service_yaml_request(url, delete=True)
+        return output["command_status"], output["command_output"]
+
+    def set_password(self, user_name, password):
+        "Set a user's password"
+        url = "json/user/name/%s" % user_name
+        post_data = { "password": password }
+        output = self.service_yaml_request(url, post_data=post_data)
+        return output["command_status"], output["command_output"]
+
     def login(self, password):
         '''Log in to the web server. Saves a sessionid
         password -- cleartext password to use to authenticate
         '''
         data = {"username": self.username, "password": password}
+        msg = "Logging in as %s to %s"
+        msg = msg  % (self.username, self.address)
+        print msg
         response = self.post(LOGIN_PATH, data)
         return_data = response.convert_from_yaml()
         self.password = password
         return return_data.get("super_user")
 
+    def package_build_job(self, package_name, svn_user, svn_password, bg_flag):
+        "start a package-building job and watch its progress"
+        url = "json/package_build/%s" % package_name
+        post_data = {"svn_user": svn_user,
+                     "svn_password": svn_password,
+                     "debug": True,
+                     "prepare": True}
+        job_name = self.get_job(url, post_data)
+        if bg_flag:
+            return OK, ["Started job: %s" % job_name]
+        return self.watch_jobs([job_name])
+
     def service_request(self, path, args=None,
                         put_data=None, post_data=None,
                         delete=False, timeout=None):
@@ -244,7 +312,10 @@ class CnmConnector:
         '''
         if not args:
             args = {}
+
         full_path = urlparse.urljoin(self.address, path)
+        #full_path = '/'.join(self.address, path)
+
         query_string = make_query_string(args)
         url = urlparse.urljoin(full_path, query_string)
         if self.debug:
@@ -274,9 +345,15 @@ class CnmConnector:
                 print "POSTING:", encoded_post_data
         return self.perform_request(curl_obj, full_path)
 
+    def get_server_config(self):
+        "Get server-side configuration data"
+        url = 'json/server/config'
+        output = self.service_yaml_request(url)
+        return output
+
     def sync_server_home(self, server_home_path):
         "For setting the base location of all data on the CNM"
-        url = '/json/server/config'
+        url = 'json/server/config'
         post_data = {"server_home": server_home_path}
         output = self.service_yaml_request(url, post_data=post_data)
         new_path = output["server_home"]
@@ -296,11 +373,81 @@ class CnmConnector:
                     put_data = yaml.dump(put_data)
             response = self.service_request(path, args, put_data, post_data,
                                             delete, timeout)
-            return response.convert_from_yaml()
+            try:
+                return response.convert_from_yaml()
+            except UnexpectedDataException, err:
+                err.url = path
+                raise err
         except urllib2.HTTPError:
             print "Unable to connect to the service %s" % path
             return {}
 
+    # MACHINE stuff:
+
+    def machine_job(self, machine_name, command, bg_flag, post_data = None):
+        "Run a non-package job on a remote machine"
+        if post_data == None:
+            post_data = {}
+        url = URL_LOOKUP[command] % machine_name
+        job_name = self.get_job(url, post_data)
+        if bg_flag:
+            return OK, ["Started job: %s" % job_name]
+        libUi.info("Watching job progress. Press ^C to abort or disconnect.")
+        return self.watch_jobs([job_name])
+
+    def setup_command(self, machine_name, password, bg_flag):
+        "Set a machine up"
+        post_data = {"yaml": yaml.dump( {"password" : password } )}
+        return self.machine_job(machine_name, "setup", bg_flag, post_data)
+
+    def enable_command(self, machine_name, password, bg_flag):
+        "Transfer ssh keys"
+        post_data = {"yaml": yaml.dump( {"password" : password } )}
+        return self.machine_job(machine_name, "enable", bg_flag, post_data)
+
+    def dist_command(self, machine_name, dist_name, bg_flag):
+        "Set up a dist on a remote machine"
+        post_data = {"dist": dist_name}
+        return self.machine_job(machine_name, "dist", bg_flag, post_data)
+
+    def package_command(self, command, machine_name, package_name, bg_flag):
+        "Watch a package-job run on a machine"
+        url = "json/package_action/%s" % package_name
+        post_data = {"machine": machine_name,
+                     "action": command}
+        job_name = self.get_job(url, post_data)
+        if bg_flag:
+            return OK, ["Started job: %s" % job_name]
+        return self.watch_jobs([job_name])
+
+    def machine_command(self, url_base, machine_name, bg_flag):
+        "Perform a simple command on a remote machine"
+        url = url_base + "/" + machine_name
+        post_data = {"machine": machine_name}
+        job_name = self.get_job(url, post_data)
+        if bg_flag:
+            return OK, ["Started job: %s" % job_name]
+        return self.watch_jobs([job_name])
+
+    def push_machine_config(self, machine_name, bg_flag):
+        "Push the configuration to a remote machine"
+        return self.machine_command("json/machine/push", machine_name, bg_flag)
+
+    def unpush_machine_config(self, machine_name, bg_flag):
+        "Remove the configuration on a remote machine"
+        return self.machine_command("json/machine/unpush",
+                                    machine_name, bg_flag)
+
+    # END OF MACHINE STUFF
+
+    def clear_broken_jobs(self, machine_name):
+        """Remove all entries in the dispatcher that show broken
+        jobs on a machine"""
+        url = "json/machine/clear-broken-jobs/"
+        post_data = {"machine": machine_name}
+        output = self.service_yaml_request(url, post_data=post_data)
+        return output["command_status"], output["command_output"]
+
     def cleanup_connections(self):
         "Call the ReST interface for clearing connections no a machine"
         url = "json/machine/cleanup_connections"
@@ -313,6 +460,13 @@ class CnmConnector:
                 return True
         return False
 
+    def kill_job(self, job_name):
+        "Kill a running job on the server or one yet to run"
+        post_data = {"post_data": "hello"}
+        url = "json/job/kill/%s" % job_name
+        output = self.service_yaml_request(url, post_data=post_data)
+        return output["command_status"], output["command_output"]
+
     def watch_jobs(self, job_names):
         "Given a list of jobs, watch their progress."
         summary_output = []
@@ -370,16 +524,21 @@ class CnmConnector:
                     machine_names.update([machine_name])
                 output = {}
                 for machine_name in machine_names:
-                    url = "/json/machine/stop-jobs/"
-                    post_data = {"machine": machine_name}
+                    _status, output = self.stop_jobs(machine_name)
                     print ">>> Halting all jobs on %s..." % machine_name
-                    output = self.service_yaml_request(url, post_data=post_data)
-                    libUi.user_output(output["command_output"], FAIL)
+                    libUi.user_output(output, FAIL)
                 raise exceptions.KeyboardInterrupt
 
         libUi.info("Finishing processing of %d jobs..." % len(jobs))
         return summary_status, summary_output
 
+    def stop_jobs(self, machine_name):
+        "Stop all the jobs on a machine"
+        url = "json/machine/stop-jobs/"
+        post_data = {"machine": machine_name}
+        output = self.service_yaml_request(url, post_data=post_data)
+        return output["command_status"], output["command_output"]
+
     def get_job(self, url, post_data):
         "Given an action dictated by URL, get a job_name that we can track"
         out = self.service_yaml_request(url, post_data=post_data)
@@ -398,40 +557,39 @@ class CnmConnector:
             raise MachineTraceback(url, output["traceback"])
         return output
 
-    def set_password(self, password):
+    def show_jobs(self, machine_name):
+        "Show what jobs are assigned to a given machine"
+        url = "json/machine/show-jobs/%s" % machine_name
+        output = self.service_yaml_request(url)
+        return output
+
+    def set_configuration_key(self, password):
         "Set the configuration-key"
-        url = "/json/dispatcher/set-password"
+        url = "json/dispatcher/set-password"
         post_data = {"password": password}
         output = self.service_yaml_request(url, post_data=post_data)
         return output["command_status"], output["command_output"]
 
-    def dispatcher_control(self, action, post_data = {}):
-        "Performing actions on the dispatcher"
-        dispatcher_url = "/json/dispatcher/%s" % action
-        if action == "status":
-            output = self.service_yaml_request(dispatcher_url)
-            if output.get('command_status') == OK:
-                return OK, output
-            else:
-                return_list = ["Dispatcher is offline."]
-                return_status = FAIL
-            return return_status, return_list
-        else:
-            try:
-                output = self.service_yaml_request(dispatcher_url, 
-                                                   post_data = post_data,
-                                                   timeout=5)
-            except ServerException:
-                if action == "start":
-                    output = {"command_status": OK,
-                              "command_output": ['Dispatcher started']}
-                else:
-                    raise
+    def dispatcher_status(self):
+        "Find out how the dispatcher's doing"
+        dispatcher_url = "json/dispatcher/"
+        output = self.service_yaml_request(dispatcher_url)
+        if output.get('command_status') == OK:
+            return OK, output
+        return FAIL, ["Dispatcher is offline."]
+
+    def dispatcher_attach(self):
+        "Attach to a running dispatcher"
+        post_data = NULL_POST
+        dispatcher_url = "json/dispatcher/"
+        output = self.service_yaml_request(dispatcher_url, 
+                                           post_data = post_data,
+                                           timeout=5)
         return output["command_status"], output["command_output"]
 
     def get_uncommented_jobs(self):
         "get a list of tuples of jobs that need commenting"
-        url = '/json/job/comment/pending/'
+        url = 'json/job/comment/pending/'
         content_dict = self.service_yaml_request(url)
         return content_dict["job_info"]
 
@@ -442,7 +600,7 @@ class CnmConnector:
                 "publish": publish,
                }
         post_data = {"yaml": yaml.dump(data)}
-        url = '/json/job/comment/pending/'
+        url = 'json/job/comment/pending/'
         content_dict = self.service_yaml_request(url, post_data=post_data)
         return content_dict
 
